@@ -5,6 +5,8 @@ All classes here inherit from `projectal.entity.Entity`.
 """
 
 from projectal.entities.access_policy import AccessPolicy
+from projectal.entities.activity import Activity
+from projectal.entities.booking import Booking
 from projectal.entities.calendar import Calendar
 from projectal.entities.company import Company
 from projectal.entities.contact import Contact
