@@ -1,3 +1,5 @@
+import copy
+
 from projectal.errors import UnsupportedException
 
 from projectal.entity import Entity
@@ -54,7 +56,7 @@ class Calendar(Entity):
         if isinstance(holder, Entity):
             cl = holder.get('calendarList', [])
             cl.append(out)
-            holder.set_readonly('calendarList', cl)
+            holder.set_readonly('calendarList', copy.deepcopy(cl))
         return out
 
     @classmethod
