@@ -3,13 +3,13 @@ from projectal import api
 from projectal.linkers import *
 
 
-class Contact(Entity, CompanyLinker, CustomerLinker, NoteLinker):
+class Contact(Entity, CompanyLinker, CustomerLinker, NoteLinker, TagLinker):
     """
     Implementation of the [Contact](https://projectal.com/docs/latest/#tag/Contact) API.
     """
     _path = 'contact'
     _name = 'contact'
-    _links = [NoteLinker]
+    _links = [NoteLinker, TagLinker]
     _links_reverse = [CompanyLinker, CustomerLinker]
 
     @classmethod
