@@ -3,13 +3,14 @@ from projectal import api
 from projectal.linkers import *
 
 
-class Department(Entity, StaffLinker, DepartmentLinker, CompanyLinker, NoteLinker):
+class Department(Entity, StaffLinker, DepartmentLinker, CompanyLinker, NoteLinker,
+                 TagLinker):
     """
     Implementation of the [Department](https://projectal.com/docs/latest/#tag/Department) API.
     """
     _path = 'department'
     _name = 'department'
-    _links = [StaffLinker, DepartmentLinker, NoteLinker]
+    _links = [StaffLinker, DepartmentLinker, NoteLinker, TagLinker]
     _links_reverse = [CompanyLinker]
 
     @staticmethod
