@@ -6,13 +6,13 @@ from projectal.linkers import *
 
 
 class Location(Entity, CompanyLinker, CustomerLinker, ProjectLinker, StaffLinker,
-               NoteLinker, CalendarLinker):
+               NoteLinker, CalendarLinker, TagLinker):
     """
     Implementation of the [Location](https://projectal.com/docs/latest/#tag/Location) API.
     """
     _path = 'location'
     _name = 'location'
-    _links = [NoteLinker, CalendarLinker]
+    _links = [NoteLinker, CalendarLinker, TagLinker]
     _links_reverse = [CompanyLinker, CustomerLinker, ProjectLinker, StaffLinker]
 
     def calendar(self, begin=None, until=None):
