@@ -1,3 +1,5 @@
+import copy
+
 import projectal
 from projectal.entity import Entity
 from projectal.linkers import *
@@ -27,7 +29,7 @@ class Note(Entity, TagLinker):
         if isinstance(holder, Entity):
             nl = holder.get('noteList', [])
             nl.append(out)
-            holder.set_readonly('noteList', nl)
+            holder.set_readonly('noteList', copy.deepcopy(nl))
 
         # Notes contain references to the objects that made and hold them. We don't get this
         # information from the creation api method, but we can insert them ourselves because
