@@ -3,18 +3,26 @@ from projectal.entity import Entity
 from projectal.linkers import *
 
 
-class User(Entity, AccessPolicyLinker, PermissionLinker, NoteLinker):
+class User(Entity, AccessPolicyLinker, PermissionLinker, NoteLinker, TagLinker):
     """
     Implementation of the [User](https://projectal.com/docs/latest/#tag/User) API.
     """
     _path = 'user'
     _name = 'user'
-    _links = [AccessPolicyLinker, PermissionLinker, NoteLinker]
+    _links = [AccessPolicyLinker, PermissionLinker, NoteLinker, TagLinker]
 
-    @classmethod
-    def register(cls, user):
-        url = '/api/user/registration/{}'.format(user['uuId'])
-        api.post(url, is_json=False)
+    def register(self):
+        url = '/api/user/registration/{}'.format(self['uuId'])
+        return api.post(url)
+
+    def set_password(self, password, token_id):
+        url = '/auth/register/password'
+        payload = {
+            'password': password,
+            'confirm': password,
+            'tokenId': token_id
+        }
+        api.post(url, payload=payload)
         return True
 
     @classmethod
@@ -24,18 +32,21 @@ class User(Entity, AccessPolicyLinker, PermissionLinker, NoteLinker):
         """
         return api.get('/api/user/permissions')['permissions']
 
-    def bulk_user_link_permission(self, permission):
-        self.__permission(self, permission, 'add')
+    def link_permission(self, permissions):
+        self.__permission(self, permissions, 'add')
 
-    def bulk_user_unlink_permission(self, permission):
-        self.__permission(self, permission, 'delete')
+    def unlink_permission(self, permissions):
+        self.__permission(self, permissions, 'delete')
 
     @classmethod
-    def __permission(cls, from_user, to_permission, operation):
+    def __permission(cls, from_user, to_permissions, operation):
+        if isinstance(to_permissions, dict):
+            to_permissions = [to_permissions]
+
         url = '/api/user/permission/{}'.format(operation)
         payload = [{
             'uuId': from_user['uuId'],
-            'permissionList': [to_permission]
+            'permissionList': to_permissions
         }]
         api.post(url, payload=payload)
         return True
