@@ -36,3 +36,18 @@ class Webhook(Entity):
         url = '/api/webhook/list?start={}&limit={}&ksort={}&order={}'.\
                   format(start, limit, ksort, order)
         return api.get(url)
+
+    @classmethod
+    def list_events(cls, **kwargs):
+        """
+        Get a list of webhook events.
+        Use parameter format=False to return eventTime as UTC timestamp.
+        """
+
+        url = '/api/webhookevent/list'
+
+        params = [f'{k}={v}' for k, v in kwargs.items()]
+        if len(params) > 0:
+            url += '?' + '&'.join(params)
+        response = api.get(url)
+        return response
