@@ -87,15 +87,15 @@ class Entity(dict):
         self._link_def_by_name[d['name']] = d
 
     def _add_link(self, to_entity_name, to_link):
-        self._link(to_entity_name, to_link, 'add')
+        self._link(to_entity_name, to_link, 'add', batch_linking=False)
 
-    def _update_link(self,to_entity_name, to_link):
-        self._link(to_entity_name, to_link, 'update')
+    def _update_link(self, to_entity_name, to_link):
+        self._link(to_entity_name, to_link, 'update', batch_linking=False)
 
     def _delete_link(self, to_entity_name, to_link):
-        self._link(to_entity_name, to_link, 'delete')
+        self._link(to_entity_name, to_link, 'delete', batch_linking=False)
 
-    def _link(self, to_entity_name, to_link, operation, update_cache=True):
+    def _link(self, to_entity_name, to_link, operation, update_cache=True, batch_linking=True):
         """
         `to_entity_name`: Destination entity name (e.g. 'staff')
 
@@ -103,9 +103,14 @@ class Entity(dict):
 
         `operation`: `add`, `update`, `delete`
 
-        update_cache: also modify the entity's internal representation of the links
+        'update_cache': also modify the entity's internal representation of the links
         to match the operation that was done. Set this to False when replacing the
         list with a new one (i.e., when calling save() instead of a linker method).
+
+        'batch_linking': Enabled by default, batches any link
+        updates required into composite API requests. If disabled
+        a request will be executed for each link update.
+        Recommended to leave enabled to increase performance.
         """
 
         link_def = self._link_def_by_name[to_entity_name]
@@ -133,10 +138,13 @@ class Entity(dict):
         if not to_link:
             return
 
+        url = ''
+        payload = {}
+        request_list = []
         # Is it a reverse linker? If so, invert the relationship
         if link_def['reverse']:
             for link in to_link:
-                link._link(self._name, self, operation, update_cache)
+                request_list.extend(link._link(self._name, self, operation, update_cache, batch_linking=batch_linking))
         else:
             # Only keep UUID and the data attribute, if it has one
             def strip_payload(link):
@@ -146,23 +154,47 @@ class Entity(dict):
                     single[data_name] = copy.deepcopy(link[data_name])
                 return single
 
+            # If batch linking is enabled and the entity to link is a list of entities,
+            # a separate request must be constructed for each one because the final composite
+            # request permits only one input per call
+            url = '/api/{}/link/{}/{}'.format(self._path, to_entity_name, operation)
             to_link_payload = None
             if isinstance(to_link, list):
                 to_link_payload = []
                 for link in to_link:
-                    to_link_payload.append(strip_payload(link))
+                    if batch_linking:
+                        request_list.append({
+                            'method': "POST",
+                            'invoke': url,
+                            'body': {
+                                'uuId': self['uuId'],
+                                to_key: [strip_payload(link)],
+                            }
+                        })
+                    else:
+                        to_link_payload.append(strip_payload(link))
             if isinstance(to_link, dict):
-                to_link_payload = strip_payload(to_link)
+                if batch_linking:
+                    request_list.append({
+                        'method': "POST",
+                        'invoke': url,
+                        'body': {
+                            'uuId': self['uuId'],
+                            to_key: strip_payload(to_link),
+                        }
+                    })
+                else:
+                    to_link_payload = strip_payload(to_link)
 
-            payload = {
-                'uuId': self['uuId'],
-                to_key: to_link_payload
-            }
-            url = '/api/{}/link/{}/{}'.format(self._path, to_entity_name, operation)
-            api.post(url, payload=payload)
+            if not batch_linking:
+                payload = {
+                    'uuId': self['uuId'],
+                    to_key: to_link_payload
+                }
+                api.post(url, payload=payload)
 
         if not update_cache:
-            return
+            return request_list
 
         # Set the initial state if first add. We need the type to be set to correctly update the cache
         if operation == 'add' and self.get(to_key, None) is None:
@@ -200,10 +232,12 @@ class Entity(dict):
         if to_key in self:
             self.__old[to_key] = self[to_key]
 
+        return request_list
+
     # -----
 
     @classmethod
-    def create(cls, entities, params=None):
+    def create(cls, entities, params=None, batch_linking=True):
         """
         Create one or more entities of the same type. The entity
         type is determined by the subclass calling this method.
@@ -214,6 +248,11 @@ class Entity(dict):
         `params`: Optional URL parameters that may apply to the
         entity's API (e.g: `?holder=1234`).
 
+        'batch_linking': Enabled by default, batches any link
+        updates required into composite API requests. If disabled
+        a request will be executed for each link update.
+        Recommended to leave enabled to increase performance.
+
         If input was a `dict`, returns an entity subclass. If input was
         a list of `dict`s, returns a list of entity subclasses.
 
@@ -274,8 +313,18 @@ class Entity(dict):
                 objects.append(orig)
 
         # Detect and apply any link additions
+        # if batch_linking is enabled, builds a list of link requests
+        # needed for each entity, then executes them with composite
+        # API requests
+        link_request_batch = []
         for e in e_list:
-            e.__apply_link_changes()
+            requests = e.__apply_link_changes(batch_linking=batch_linking)
+            link_request_batch.extend(requests)
+
+        if len(link_request_batch) > 0 and batch_linking:
+            for i in range(0, len(link_request_batch), 100):
+                chunk = link_request_batch[i:i + 100]
+                api.post('/api/composite', chunk)
 
         if not isinstance(entities, list):
             return objects[0]
@@ -300,7 +349,7 @@ class Entity(dict):
         return link_set
 
     @classmethod
-    def get(cls, entities, links=None):
+    def get(cls, entities, links=None, deleted_at=None):
         """
         Get one or more entities of the same type. The entity
         type is determined by the subclass calling this method.
@@ -340,6 +389,9 @@ class Entity(dict):
         # str with links
         projectal.Project.get('1b21e445-f29a...', 'links=['company', 'location']')
         ```
+
+        `deleted_at`: Include this parameter to get a deleted entity.
+        This value should be a UTC timestamp from a webhook delete event.
         """
         link_set = cls._get_linkset(links)
 
@@ -361,9 +413,17 @@ class Entity(dict):
             # We have a list of dicts already, the expected format
             payload = entities
 
+        if deleted_at:
+            if not isinstance(deleted_at, int):
+                raise projectal.UsageException("deleted_at must be a number")
+
         url = '/api/{}/get'.format(cls._path)
-        if links:
-            url += '?links={}'.format(','.join(links))
+        params = []
+        params.append('links={}'.format(','.join(links))) if links else None
+        params.append('epoch={}'.format(deleted_at - 1)) if deleted_at else None
+        if len(params) > 0:
+            url += '?' + '&'.join(params)
+
         # We only need to send over the uuIds
         payload = [{'uuId': e['uuId']} for e in payload]
         if not payload:
@@ -396,7 +456,7 @@ class Entity(dict):
         return super(Entity, self).get(*args, **kwargs)
 
     @classmethod
-    def update(cls, entities):
+    def update(cls, entities, batch_linking=True):
         """
         Save one or more entities of the same type. The entity
         type is determined by the subclass calling this method.
@@ -406,6 +466,11 @@ class Entity(dict):
         `entities`: Can be a `dict` to update a single entity,
         or a list of `dict`s to update many entities in bulk.
 
+        'batch_linking': Enabled by default, batches any link
+        updates required into composite API requests. If disabled
+        a request will be executed for each link update.
+        Recommended to leave enabled to increase performance.
+
         Returns `True` if all entities update successfully.
 
         ```
@@ -440,9 +505,20 @@ class Entity(dict):
                 api.put('/api/{}/update'.format(cls._path), chunk)
 
         # Detect and apply any link changes
+        # if batch_linking is enabled, builds a list of link requests
+        # from the changes of each entity, then executes
+        # composite API requests with those changes
+        link_request_batch = []
         for e in e_list:
             if isinstance(e, Entity):
-                e.__apply_link_changes()
+                requests = e.__apply_link_changes(batch_linking=batch_linking)
+                link_request_batch.extend(requests)
+
+        if len(link_request_batch) > 0 and batch_linking:
+            for i in range(0, len(link_request_batch), 100):
+                chunk = link_request_batch[i:i + 100]
+                api.post('/api/composite', chunk)
+
         return True
 
     def __update(self, *args, **kwargs):
@@ -701,19 +777,75 @@ class Entity(dict):
     def changes(self):
         """Return a dict containing the fields that have changed since fetching the object.
         Dict values contain both the old and new values. E.g.: {'old': 'original', 'new': 'current'}.
+
+        In the case of link lists, there are three values: added, removed, updated. Only links with
+        a data attribute can end up in the updated list, and the old/new dictionary is placed within
+        that data attribute. E.g. for a staff-resource link:
+        'updated': [{
+            'uuId': '24eb4c31-0f92-49d1-8b4d-507ab939003e',
+            'resourceLink': {'quantity': {'old': 2, 'new': 5}}
+        }]
         """
         changed = {}
         for key in self.keys():
             link_def = self._link_def_by_key.get(key)
             if link_def:
-                if self.__apply_list(link_def, report_only=True):
-                    changed[key] = {'old': self.__old.get(key), 'new': self[key]}
+                changes = self._changes_for_link_list(link_def, key)
+                # Only add it if something in it changed
+                for action in changes.values():
+                    if len(action):
+                        changed[key] = changes
+                        break
             elif key not in self.__old and self[key] is not None:
                 changed[key] = {'old': None, 'new': self[key]}
             elif self.__old.get(key) != self[key]:
                 changed[key] = {'old': self.__old.get(key), 'new': self[key]}
         return changed
 
+    def _changes_for_link_list(self, link_def, key):
+        changes = self.__apply_list(link_def, report_only=True)
+        data_key = link_def['data_name']
+
+        # For linked entities, we will only report their UUID, name (if it has one),
+        # and the content of their data attribute (if it has one).
+        def get_slim_list(entities):
+            slim = []
+            if isinstance(entities, dict):
+                entities = [entities]
+            for e in entities:
+                fields = {'uuId': e['uuId']}
+                name = e.get('name')
+                if name:
+                    fields['name'] = e['name']
+                if data_key and e[data_key]:
+                    fields[data_key] = e[data_key]
+                slim.append(fields)
+            return slim
+
+        out = {
+            'added': get_slim_list(changes.get('add', [])),
+            'updated': [],
+            'removed': get_slim_list(changes.get('remove', [])),
+        }
+
+        updated = changes.get('update', [])
+        if updated:
+            before_map = {}
+            for entity in self.__old.get(key):
+                before_map[entity['uuId']] = entity
+
+            for entity in updated:
+                old_data = before_map[entity['uuId']][data_key]
+                new_data = entity[data_key]
+                diff = {}
+                for key in new_data.keys():
+                    if key not in old_data and new_data[key] is not None:
+                        diff[key] = {'old': None, 'new': new_data[key]}
+                    elif old_data.get(key) != new_data[key]:
+                        diff[key] = {'old': old_data.get(key), 'new': new_data[key]}
+                out['updated'].append({'uuId': entity['uuId'], data_key: diff})
+        return out
+
     def _changes_internal(self):
         """Return a dict containing only the fields that have changed and their current value,
         without any link data.
@@ -752,7 +884,7 @@ class Entity(dict):
         # Links without data never differ
         return False
 
-    def __apply_link_changes(self):
+    def __apply_link_changes(self, batch_linking=True):
         """Send each link list to the conflict resolver. If we detect
         that the entity was not fetched with that link, we do the fetch
         first and use the result as the basis for comparison."""
@@ -774,10 +906,16 @@ class Entity(dict):
             for _list in find_list:
                 self.__old[_list] = copy.deepcopy(new.get(_list, []))
 
-        for keys in self._link_def_by_key.values():
-            self.__apply_list(keys)
+        # if batch_linking is enabled, builds a list of link requests
+        # for each link definition of the calling entity then returns the list
+        request_list = []
+        for link_def in self._link_def_by_key.values():
+            link_def_requests = self.__apply_list(link_def, batch_linking=batch_linking)
+            if batch_linking:
+                request_list.extend(link_def_requests)
+        return request_list
 
-    def __apply_list(self, link_def, report_only=False):
+    def __apply_list(self, link_def, report_only=False, batch_linking=True):
         """Automatically resolve differences and issue the correct sequence of
         link/unlink/relink for the link list to result in the supplied list
         of entities.
@@ -841,15 +979,30 @@ class Entity(dict):
             # Would be an error in this library if we reach here
             raise projectal.UnsupportedException("This type does not support linking")
 
+        # if batch_linking is enabled, builds a list of requests
+        # from each link method
         if not report_only:
+            request_list = []
             if to_remove:
-                self._link(link_def['name'], to_remove, 'delete', update_cache=False)
+                delete_requests = self._link(
+                        link_def['name'], to_remove, 'delete',
+                        update_cache=False, batch_linking=batch_linking
+                )
+                request_list.extend(delete_requests)
             if to_update:
-                self._link(link_def['name'], to_update, 'update', update_cache=False)
+                update_requests = self._link(
+                    link_def['name'], to_update, 'update',
+                    update_cache=False, batch_linking=batch_linking
+                )
+                request_list.extend(update_requests)
             if to_add:
-                self._link(link_def['name'], to_add, 'add', update_cache=False)
-
+                add_requests = self._link(
+                    link_def['name'], to_add, 'add',
+                    update_cache=False, batch_linking=batch_linking
+                )
+                request_list.extend(add_requests)
             self.__old[link_key] = copy.deepcopy(want_entities)
+            return request_list
         else:
             changes = {}
             if to_remove:
