@@ -30,6 +30,7 @@ task.link_staff(staff)
 
 from projectal import api
 
+
 class BaseLinker:
     # _link_name is the link name (usually the entity name)
     _link_name = None
@@ -56,6 +57,7 @@ class BaseLinker:
     # name. E.g: stage_list needs to be converted to Stage.
     _link_entity = None
 
+
 class AccessPolicyLinker(BaseLinker):
     """Subclass can link to Access Policies"""
     _link_name = 'access_policy'
@@ -69,6 +71,28 @@ class AccessPolicyLinker(BaseLinker):
         self._delete_link('access_policy', access_policies)
 
 
+class ActivityLinker(BaseLinker):
+    """Subclass can link to Activities"""
+    _link_name = 'activity'
+
+    def link_activity(self, activity):
+        self._add_link('activity', activity)
+
+    def unlink_activity(self, activity):
+        self._delete_link('activity', activity)
+
+
+class BookingLinker(BaseLinker):
+    """Subclass can link to Bookings"""
+    _link_name = 'booking'
+
+    def link_booking(self, booking):
+        self._add_link('booking', booking)
+
+    def unlink_booking(self, booking):
+        self._delete_link('booking', booking)
+
+
 class CompanyLinker(BaseLinker):
     """Subclass can link to Companies"""
     _link_name = 'company'
@@ -184,7 +208,7 @@ class RebateLinker(BaseLinker):
 class ResourceLinker(BaseLinker):
     """Subclass can link to Resources"""
     _link_name = 'resource'
-    _link_data_name =  'resourceLink'
+    _link_data_name = 'resourceLink'
 
     def link_resource(self, resources):
         self._add_link('resource', resources)
@@ -199,7 +223,7 @@ class ResourceLinker(BaseLinker):
 class SkillLinker(BaseLinker):
     """Subclass can link to Skills"""
     _link_name = 'skill'
-    _link_data_name =  'skillLink'
+    _link_data_name = 'skillLink'
 
     def link_skill(self, skills):
         self._add_link('skill', skills)
@@ -214,7 +238,7 @@ class SkillLinker(BaseLinker):
 class StaffLinker(BaseLinker):
     """Subclass can link to Staff"""
     _link_name = 'staff'
-    _link_data_name =  'resourceLink'
+    _link_data_name = 'resourceLink'
 
     def link_staff(self, staffs):
         self._add_link('staff', staffs)
